!DIR$ FIXEDFORMLINESIZE:132
      include 'dblocks.f'
      include 'formats.f'
      include 'values.f'
      program RectDisp
      use dblocks
      use formats
      use values
      implicit none

      complex*16  cqz
      complex*16, external :: cfun_i, cfun_h
      real*8     msum, eps_re, eps_im, rk_i, rk_h
      real*4     arg
      integer*4  ip, Mode, info, info_1, info_2
      character*4 answ

      external check_pars, make_M, make_Mode, find_qz

      call check_pars(info)
      if     (info .eq. 0) then

         write(6,f1) '>  RectDisp : Input pars have been checked for sanity'

      else if(info .gt. 0) then

         write(6,f2) '>  RectDisp : Errors on input; info=', info
         stop

      else if(info .lt. 0) then

         write(6,f3) '>  RectDisp : Warnings on input; info=', info, '; proceed anyway? [y/n]'

 1       continue
         read(*,*) answ

         if     ((answ .eq. 'y') .or. (answ .eq. 'Y')) then
            continue
         else if((answ .eq. 'n') .or. (answ .eq. 'N')) then
            stop
         else
            write(6,f1) '>  RectDisp : Incorrect input, please repeat. Proceed anyway? [y/n]'
            goto 1
         end if

      end if

      write(6,f4) '>  RectDisp : Inclusion volume fraction,   rho=', sngl(rho)

      allocate(Mx(-Lx2:Lx2))
      allocate(My(-Ly2:Ly2))
      allocate(Mz(-Lz2:Lz2))
      call make_M(msum)
      write(6,f4) '>  RectDisp : Matrices M have been built, msum=', sngl(msum)

      call make_Mode(Mode)
      if(Mode .lt. 0) then
         write(6,f2) '>  RectDisp : Incorrect or ambiguous polarization, Mode=', Mode
         stop
      else if(Mode .eq. 7) then
         write(6,f2) '>  RectDisp : Mode=7 for general polarization not implemented; exiting'
         stop
      else
         write(6,f2) '>  RectDisp : Polarization mode defined, Mode=', Mode
      end if

c     
c--------- Frequency scan -----------------------------------------------------
c
      open(unit=71, file='disp.dat', status='unknown')
      open(unit=72, file='conv.dat', status='unknown')
      if     (any(PERM_i .eq. Perm_dsp)) then
         open(unit=73, file='eps_i.dat', status='unknown')
      else if(any(PERM_i .eq. Perm_fil)) then
         open(unit=73, file='eps_i.dat', status='old',err=101)
      end if
      if     (any(PERM_h .eq. Perm_dsp)) then
         open(unit=74, file='eps_h.dat', status='unknown')
      else if(any(PERM_h .eq. Perm_fil)) then
         open(unit=74, file='eps_h.dat', status='old',err=102)
      end if

      do 10, ip=1,Np
         
      if(any(PERM_i .eq. Perm_fil) .and. any(PERM_h .eq. Perm_fil)) then
         read(73,*,err=103,end=103) rk_i, eps_re, eps_im
         ceps_i = eps_re + cu*eps_im
         read(74,*,err=104,end=104) rk_h, eps_re, eps_im
         ceps_h = eps_re + cu*eps_im
         if(rk_i .ne. rk_h) then
            write(6,f1) '>  RectDisp : Frequency values in "eps_i.dat" and "eps_h.dat" are inconsistent; exiting"'
            stop
         end if
         rk = rk_i
      else if(any(PERM_i .eq. Perm_fil)) then
         read(73,*,err=103,end=103) rk, eps_re, eps_im
         ceps_i = eps_re + cu*eps_im
      else if(any(PERM_h .eq. Perm_fil)) then
         read(74,*,err=104,end=104) rk, eps_re, eps_im
         ceps_h = eps_re + cu*eps_im
      else
         rk  = rk_min + drk*dfloat(ip-1)
         if(rk .lt. qthr) rk = qthr
      end if

      arg = sngl(rk)

      if     (any(PERM_i .eq. Perm_fix)) then
         ceps_i = ceps_i_fix
      else if(any(PERM_i .eq. Perm_dsp)) then
         ceps_i = cfun_i(rk)
         write(73,*) arg, sngl(dreal(ceps_i)), sngl(dimag(ceps_i))
      else if(any(PERM_i .eq. Perm_fil)) then
         continue
      else
         write(6,f1) '>  RectDisp : Unexpected error at Point A [incorrect PERM_i]; exiting'
         stop
      end if

      if     (any(PERM_h .eq. Perm_fix)) then
         ceps_h = ceps_h_fix
      else if(any(PERM_h .eq. perm_dsp)) then
         ceps_h = cfun_h(rk)
         write(74,*) arg, sngl(dreal(ceps_h)), sngl(dimag(ceps_h))
      else if(any(PERM_h .eq. Perm_fil)) then
         continue
      else
         write(6,f1) '>  RectDisp : Unexpected error at Point B [incorrect PERM_h]; exiting'
         stop
      end if

      ceps_a = rho*ceps_i + (ru-rho)*ceps_h

      rk  = pi*rk/hz
      rk2 = rk*rk
      rkx = fkx*rk
      rky = fky*rk

      chi  = rho*(ceps_i - ceps_h)/ceps_h
      ckh2 = rk2*ceps_h 

      call find_qz(Mode, cqz, info, info_1, info_2)
      if(info .gt. 0) then
         write(71,*) arg,  sngl(dreal(cqz)/fbz),  sngl(dimag(cqz*hz))
         write(71,*) arg, -sngl(dreal(cqz)/fbz), -sngl(dimag(cqz*hz))
      end if
      write(72,*) arg, info, info_1, info_2

      write(6, 400, advance='NO') CC, '------ Complete : ', ip , ' of ', Np
      flush(6)

 10   continue

      close(unit=71)
      close(unit=72)
      if(any(PERM_i .eq. Perm_dsp) .or. any(PERM_i .eq. Perm_fil)) then
         close(unit=73)
      end if
      if(any(PERM_h .eq. Perm_dsp) .or. any(PERM_h .eq. Perm_fil)) then
         close(unit=74)
      end if

      write(6,f2) 
      write(6,f2) '>  RectDisp : All done!'

 400  format(a1, a18, i10, a4, i10)

      stop

 101  continue
      write(6,f1) '>  RectDisp : Unable to open required file "eps_i.dat"; exiting'
      stop

 102  continue
      write(6,f1) '>  RectDisp : Unable to open required file "eps_h.dat"; exiting'
      stop

 103  continue
      write(6,f1) '>  RectDisp : Error reading or end of file "eps_i.dat"; exiting'
      stop

 104  continue
      write(6,f1) '>  RectDisp : Error reading or end of file "eps_h.dat"; exiting'
      stop


      end

